import brainstate


class BasicModel(brainstate.nn.Dynamics):
    """
    Base class for basic models.
    """

    pass


class BasicModelGroup(brainstate.nn.DynamicsGroup):
    """
    Base class for groups of basic models.
    """

    pass
